import { h, Host } from "@stencil/core";
import { getFormElement, inputsWithAnswers, requiredChildInputs } from "../../utils/inputBase";
import { ignoreKeypress } from "../../utils/utils";
export class TaskSubmit {
    constructor() {
        this.keyboardShortcut = undefined;
        this.disabled = false;
        this.disableUntilComplete = true;
        this.label = "Submit";
        this.correctionMode = false;
        this.readyToSubmit = false;
    }
    componentWillLoad() {
        // TODO: This probably needs to be rerun if the mode changes for some reason
        this.refreshSubmitReady();
        if (this.keyboardShortcut) {
            this.registerKeyboardShortcut.emit({ label: "Submit", keys: this.keyboardShortcut });
        }
    }
    connectedCallback() {
        if (['interactive', 'complete'].includes(document.readyState)) {
            this.setupFormListener();
        }
        else {
            this.loadCallback = this.setupFormListener.bind(this);
            document.addEventListener("load", this.loadCallback);
        }
    }
    setupFormListener() {
        this.form = getFormElement(this.host);
        if (this.form) {
            this.formUpdatedCallback = this.refreshSubmitReady.bind(this);
            this.form.addEventListener("input", this.formUpdatedCallback);
            this.refreshSubmitReady();
        }
        else {
            setTimeout(() => this.setupFormListener(), 100);
        }
    }
    disconnectedCallback() {
        if (this.formUpdatedCallback) {
            this.form.removeEventListener("input", this.formUpdatedCallback);
        }
        if (this.loadCallback) {
            document.removeEventListener("load", this.loadCallback);
        }
    }
    keypressHandler(event) {
        if (!ignoreKeypress() && event.key === this.keyboardShortcut && !this.disabled) {
            this.button.click();
        }
    }
    async refreshSubmitReady() {
        if (this.form && this.disableUntilComplete) {
            Promise.all(requiredChildInputs(this.form).map(i => i.readyToSubmit()))
                .then(values => {
                this.disabled = !values.every(Boolean);
            });
        }
    }
    handleValidationResult(values) {
        if (values.every(Boolean)) {
            this.readyToSubmit = true;
            this.button.click();
        }
        else {
            this.showCorrections.emit();
            this.correctionMode = true;
        }
    }
    handleSubmit(event) {
        const answeredInputs = inputsWithAnswers();
        if (answeredInputs && answeredInputs.length > 0 && !this.readyToSubmit && !this.correctionMode) {
            event.preventDefault();
            Promise.all(answeredInputs.map(input => input.validateAgainstAnswer()))
                .then(values => this.handleValidationResult(values));
        }
        else {
            this.taskSubmit.emit();
        }
    }
    async setShowCorrections(value) {
        this.correctionMode = value;
    }
    render() {
        // Note that the button tag must be a single row to prevent crowd-form adding another one
        return (h(Host, { key: '707a0df3333865b0c88ee162427c5567169d311f' }, h("button", { key: 'd90af2681d8e7c9f09d92ade6ce66bb1f4b343ef', type: "submit", ref: el => this.button = el, disabled: this.disabled, onClick: this.handleSubmit.bind(this) }, this.label), h("div", { key: '5b5d87b901813ecfe033199fa260fbe0ebd8ce16', class: this.correctionMode ? "" : "message-hidden" }, h("slot", { key: 'd990bbd54ada648f46c38f5bc0b366c613316785' }))));
    }
    static get is() { return "task-submit"; }
    static get encapsulation() { return "scoped"; }
    static get originalStyleUrls() {
        return {
            "$": ["task-submit.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["task-submit.css"]
        };
    }
    static get properties() {
        return {
            "keyboardShortcut": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "keyboard-shortcut",
                "reflect": false
            },
            "disabled": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "disabled",
                "reflect": false,
                "defaultValue": "false"
            },
            "disableUntilComplete": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "disable-until-complete",
                "reflect": false,
                "defaultValue": "true"
            },
            "label": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "label",
                "reflect": false,
                "defaultValue": "\"Submit\""
            }
        };
    }
    static get states() {
        return {
            "correctionMode": {},
            "readyToSubmit": {}
        };
    }
    static get events() {
        return [{
                "method": "registerKeyboardShortcut",
                "name": "registerKeyboardShortcut",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "KeyboardShortcut",
                    "resolved": "{ label: string; keys: string; value?: string; }",
                    "references": {
                        "KeyboardShortcut": {
                            "location": "import",
                            "path": "../../utils/utils",
                            "id": "src/utils/utils.ts::KeyboardShortcut"
                        }
                    }
                }
            }, {
                "method": "showCorrections",
                "name": "showCorrections",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "taskSubmit",
                "name": "taskSubmit",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get methods() {
        return {
            "refreshSubmitReady": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            },
            "setShowCorrections": {
                "complexType": {
                    "signature": "(value: boolean) => Promise<void>",
                    "parameters": [{
                            "name": "value",
                            "type": "boolean",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "host"; }
    static get listeners() {
        return [{
                "name": "keydown",
                "method": "keypressHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=task-submit.js.map
