import { Host, h } from "@stencil/core";
import { getMarks, mark } from "../../utils/utils";
export class TaskTelemetry {
    constructor() {
        this.attributes = {};
        this.recordingActive = true;
        this.idle = false;
        this.lastHeartbeatTime = {};
        this.timeMeasures = [];
        this.name = "telemetry";
        this.submitEndpoint = undefined;
        this.heartbeatEndpoint = undefined;
        this.heartbeatInterval = 10;
        this.localStorageId = undefined;
        this.includeResponse = false;
        this.value = undefined;
    }
    /*
     When connected to the DOM two heartbeats are active. The internal heartbeat triggers updating the telemetry
     value and optionally writing it to localStorage. The second can send a heartbeat message to the specified
     endpoint.
     */
    connectedCallback() {
        this.internalHeartbeat = window.setInterval(() => {
            this.persistData();
        }, 2000);
        this.heartbeat = window.setInterval(() => {
            this.sendHeartbeat();
        }, this.heartbeatInterval * 1000);
    }
    disconnectedCallback() {
        window.clearInterval(this.internalHeartbeat);
        window.clearInterval(this.heartbeat);
    }
    /*
     Default listeners for expected mark events
     */
    markFocusEventHandler(event) {
        this.idle = event.type !== 'focus';
        this.markEventHandler(event);
    }
    markEventHandler(event) {
        // TODO: Find a way to address duplicate clicks being fired on radio input clicks
        mark(event.type);
    }
    /*
     This allows us to toggle to an alternate fetch method when provided
     */
    fetcher(input, init) {
        if (this.fetchMethod) {
            return this.fetchMethod(input, init);
        }
        else {
            return fetch(input, init);
        }
    }
    sendHeartbeat() {
        if (this.heartbeatEndpoint && this.recordingActive) {
            this.persistData();
            const timeDelta = subtractMeasures(this.value.seconds, this.lastHeartbeatTime);
            this.lastHeartbeatTime = this.value.seconds;
            return this.fetcher(this.heartbeatEndpoint, {
                method: "POST",
                body: JSON.stringify({
                    attributes: this.attributes,
                    timestamp: new Date(Date.now()).toISOString(),
                    idle: this.idle,
                    seconds: timeDelta
                }),
                headers: {
                    "Accept": "application/json",
                    "Content-type": "application/json"
                }
            }).catch(() => {
                console.error("Sending heartbeat failed");
            });
        }
    }
    handleSubmit() {
        this.persistData(true);
        const active = this.recordingActive;
        this.recordingActive = false;
        if (this.submitEndpoint && active) {
            if (this.includeResponse) {
                const formData = new FormData(this.input.form);
                this.value.response = Object.fromEntries(formData.entries());
                this.value.submitTimestamp = new Date(Date.now()).toISOString();
            }
            return this.fetcher(this.submitEndpoint, {
                method: "POST",
                body: JSON.stringify(this.value),
                headers: {
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                }
            }).catch(() => {
                console.error("Sending telemetry failed");
            });
        }
    }
    persistData(isSubmit = false) {
        if (this.recordingActive) {
            // Get the list of marks
            let marks = getMarks()
                .map(mark => {
                return { "name": mark.name.split(":")[1], time: Math.round(mark.startTime) };
            });
            marks = deduplicateMarks(marks);
            // console.log(marks)
            // compute time spent
            const milliseconds = {
                task: 0,
                blur: 0,
                load: 0,
                focus: 0
            };
            let lastBlurTime = 0;
            let isBlurred = false;
            const now = Math.round(window.performance.now());
            for (let m of marks) {
                switch (m.name) {
                    case "start":
                        milliseconds.task = now - m.time;
                        lastBlurTime = m.time;
                        milliseconds.load = m.time;
                        break;
                    case "focus":
                        milliseconds.blur += (m.time - lastBlurTime);
                        isBlurred = false;
                        break;
                    case "blur":
                        lastBlurTime = m.time;
                        isBlurred = true;
                        break;
                }
            }
            if (isBlurred) {
                milliseconds.blur += (now - lastBlurTime);
            }
            milliseconds.focus = milliseconds.task - milliseconds.blur;
            const seconds = {
                task: milliseconds.task / 1000,
                focus: milliseconds.focus / 1000,
                blur: milliseconds.blur / 1000,
                load: milliseconds.load / 1000
            };
            const timeMeasures = this.computeTimeMeasures(marks, isSubmit);
            const combinedSeconds = Object.assign(Object.assign({}, seconds), timeMeasures);
            // console.log(combinedSeconds)
            // count relevant marks
            const excludedMarks = ["all-crowd-elements-ready"];
            const cMarks = marks.filter(m => !excludedMarks.includes(m.name));
            const markCounts = cMarks.reduce((counts, { name }) => {
                if (name in counts) {
                    counts[name]++;
                }
                else {
                    counts[name] = 1;
                }
                return counts;
            }, {});
            // console.log(counts)
            /*
            There are two storage locations here with different purposes.
            1. The localStorage is used to capture the current state of the time
            measures and mark counts so that they can be persisted in the event
            of a restart or SMGT release/resume.
            2. The `value` state (also stored in the input element) contains the sum
            time and marks for the current session plus any priorData.
      
            To support multiple restarts, any priorData is also persisted in local
            storage.
             */
            if (this.localStorageId) {
                if (isSubmit) {
                    localStorage.removeItem(this.localStorageId);
                }
                else {
                    const data = {
                        seconds,
                        markCounts,
                        priorData: this.priorData
                    };
                    localStorage.setItem(this.localStorageId, JSON.stringify(data));
                }
            }
            this.value = {
                seconds: combinedSeconds,
                markCounts,
                totalSeconds: roundTime(this.priorData ? sumMeasures(combinedSeconds, this.priorData.seconds) : combinedSeconds),
                totalMarkCounts: this.priorData ? sumMeasures(markCounts, this.priorData.markCounts) : markCounts,
                attributes: this.attributes,
                startTimestamp: this.startTimestamp,
            };
        }
    }
    computeTimeMeasures(marks, isSubmit = false) {
        const result = {};
        let lastStart = null;
        for (let tm of this.timeMeasures) {
            let millis = 0;
            for (let mark of marks) {
                switch (mark.name) {
                    case tm.startMark:
                        lastStart = mark.time;
                        break;
                    case tm.endMark:
                        if (lastStart !== null) {
                            millis += (mark.time - lastStart);
                        }
                        break;
                }
                if (millis > 0) {
                    result[tm.name] = millis / 1000;
                }
            }
            if (lastStart !== null && ((isSubmit && tm.endMark === "submit") || tm.endMark === "now")) {
                result[tm.name] = Math.round(window.performance.now() - lastStart) / 1000;
            }
        }
        return result;
    }
    async setFetchMethod(func) {
        this.fetchMethod = func;
    }
    async mark(name) {
        mark(name);
    }
    async addTimeMeasure(name, startMark, endMark) {
        this.timeMeasures.push({ name, startMark, endMark });
    }
    componentWillLoad() {
        // Grab "priorData" from any stored values with this key
        if (this.localStorageId) {
            const priorDataString = localStorage.getItem(this.localStorageId);
            if (priorDataString) {
                this.priorData = JSON.parse(priorDataString);
                mark("restart");
                // flatten if multiple restarts
                if (this.priorData.priorData) {
                    this.priorData = {
                        seconds: sumMeasures(this.priorData.seconds, this.priorData.priorData.seconds),
                        markCounts: sumMeasures(this.priorData.markCounts, this.priorData.priorData.markCounts)
                    };
                }
            }
        }
    }
    componentDidLoad() {
        mark("start");
        this.startTimestamp = new Date(Date.now()).toISOString();
        const childAttributes = Array.from(this.host.querySelectorAll("task-telemetry-attribute").values());
        for (let child of childAttributes) {
            this.attributes[child.getAttribute("name")] = child.getAttribute("value");
        }
        this.attributes["userAgent"] = window.navigator.userAgent;
        // Delay sending the first heartbeat in case another script is setting the fetchMethod
        setTimeout(this.sendHeartbeat.bind(this), 2000);
    }
    render() {
        return (h(Host, { key: '55fa71394acf04aabc9edd19894f08de9bb44702' }, h("input", { key: '4670e8607e019bbe3983b4b3f63ad9a41331a94d', type: "hidden", name: this.name, value: JSON.stringify(this.value), ref: el => this.input = el }), h("slot", { key: '5b8c31f3553a0ec46ddfc6765343692b014cf383' })));
    }
    static get is() { return "task-telemetry"; }
    static get encapsulation() { return "scoped"; }
    static get originalStyleUrls() {
        return {
            "$": ["task-telemetry.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["task-telemetry.css"]
        };
    }
    static get properties() {
        return {
            "name": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "name",
                "reflect": false,
                "defaultValue": "\"telemetry\""
            },
            "submitEndpoint": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "submit-endpoint",
                "reflect": false
            },
            "heartbeatEndpoint": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "heartbeat-endpoint",
                "reflect": false
            },
            "heartbeatInterval": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "heartbeat-interval",
                "reflect": false,
                "defaultValue": "10"
            },
            "localStorageId": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "local-storage-id",
                "reflect": false
            },
            "includeResponse": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "include-response",
                "reflect": false,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "value": {}
        };
    }
    static get methods() {
        return {
            "setFetchMethod": {
                "complexType": {
                    "signature": "(func: Function) => Promise<void>",
                    "parameters": [{
                            "name": "func",
                            "type": "Function",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "Function": {
                            "location": "global",
                            "id": "global::Function"
                        },
                        "FetchFunction": {
                            "location": "global",
                            "id": "global::FetchFunction"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            },
            "mark": {
                "complexType": {
                    "signature": "(name: string) => Promise<void>",
                    "parameters": [{
                            "name": "name",
                            "type": "string",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            },
            "addTimeMeasure": {
                "complexType": {
                    "signature": "(name: string, startMark: string, endMark: string) => Promise<void>",
                    "parameters": [{
                            "name": "name",
                            "type": "string",
                            "docs": ""
                        }, {
                            "name": "startMark",
                            "type": "string",
                            "docs": ""
                        }, {
                            "name": "endMark",
                            "type": "string",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "host"; }
    static get listeners() {
        return [{
                "name": "focus",
                "method": "markFocusEventHandler",
                "target": "window",
                "capture": false,
                "passive": false
            }, {
                "name": "blur",
                "method": "markFocusEventHandler",
                "target": "window",
                "capture": false,
                "passive": false
            }, {
                "name": "all-crowd-elements-ready",
                "method": "markEventHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "keypress",
                "method": "markEventHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "click",
                "method": "markEventHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "submit",
                "method": "handleSubmit",
                "target": "document",
                "capture": true,
                "passive": false
            }];
    }
}
function deduplicateMarks(marks) {
    // We assume that the marks are already sorted by time
    const newMarks = [];
    let lastMark;
    for (let m of marks) {
        if (!lastMark || Math.round(lastMark.time / 10) !== Math.round(m.time / 10) || lastMark.name !== m.name) {
            newMarks.push(m);
            lastMark = m;
        }
    }
    return newMarks;
}
function sumMeasures(...objs) {
    return objs.reduce((a, b) => {
        for (let k in b) {
            if (b.hasOwnProperty(k))
                a[k] = (a[k] || 0) + b[k];
        }
        return a;
    }, {});
}
function subtractMeasures(a, b) {
    const delta = Object.assign({}, a);
    for (let k in b) {
        if (delta.hasOwnProperty(k)) {
            delta[k] = delta[k] - b[k];
        }
    }
    return delta;
}
function roundTime(seconds) {
    return Object.keys(seconds).reduce((r, k) => {
        if (seconds[k] > 0) {
            r[k] = Math.round(seconds[k] * 100) / 100;
        }
        return r;
    }, {});
}
//# sourceMappingURL=task-telemetry.js.map
