import { getAnswerCorrectionElement, getAnswerElement } from "./utils";
import { TaskInputCheckbox } from "../components/task-input-checkbox/task-input-checkbox";
export function childInputs(parent) {
    return Array.from(parent.querySelectorAll("*")).filter(element => {
        return element.nodeName.startsWith("TASK-INPUT")
            && element.nodeName !== "TASK-INPUT-OPTION";
    });
    /*.filter(element => {
    return element.parentElement.nodeName !== "TASK-ANSWER" ||
      ((element.parentElement as unknown) as TaskAnswer).displayCorrection
  }) as unknown[]) as Input[]*/
}
export function requiredChildInputs(parent) {
    return childInputs(parent).filter(input => input.required && !input.disabled && !input.hidden);
}
export function inputsWithAnswers() {
    const inputs = Array.from(document.querySelectorAll('*')).filter(element => {
        return element.nodeName.startsWith("TASK-INPUT")
            && element.nodeName !== "TASK-INPUT-OPTION"
            && element.parentElement.nodeName !== "TASK-ANSWER";
    });
    return inputs.filter(element => {
        return Boolean(element.querySelector("TASK-ANSWER"));
    });
}
function getNamedCheckbox(form, name) {
    const el = form.elements.namedItem(name);
    if (el instanceof RadioNodeList) {
        const rnl = el;
        const taskNodes = Array.from(rnl.values()).filter(r => r.nodeName === "TASK-INPUT-CHECKBOX");
        if (taskNodes.length > 0) {
            return taskNodes[0];
        }
        return undefined;
    }
    else if (el instanceof TaskInputCheckbox) {
        return el;
    }
    else {
        return undefined;
    }
}
export function evaluateFormula(formula, form) {
    const operators = ["=", " in ", " IN ", " In ", "!=", "<=", ">=", "<", ">"];
    let operator = null;
    let parts = null;
    let not = false;
    let any = false;
    if (formula.startsWith("not(") && formula.endsWith(")")) {
        not = true;
        formula = formula.substring(4, formula.length - 1);
    }
    for (let op of operators) {
        if (formula.includes(op)) {
            operator = op.toLowerCase().trim();
            parts = formula.split(op).map(p => p.trim());
            break;
        }
    }
    if (parts) {
        if (operator === "in") {
            const cb = getNamedCheckbox(form, parts[1]);
            if (cb) {
                const value = cb.value;
                if (value) {
                    const evaluation = evaluateOperator(operator, [parts[0]], value);
                    const success = evaluation.filter(e => e);
                    return success.length > 0;
                }
                return false;
            }
        }
        const inputNames = getFormulaInputNames(parts[0]);
        any = inputNames.length > 1 && parts[0].startsWith("any");
        const values = inputNames.map(name => form.elements[name] && form.elements[name].value);
        const evaluation = evaluateOperator(operator, values, parts[1]);
        const success = evaluation.filter(e => e);
        const evalTrue = success.length === inputNames.length || (any && success.length > 0);
        return (evalTrue && !not) || (!evalTrue && not);
    }
}
function evaluateOperator(op, values, formulaSuffix) {
    switch (op) {
        case "=":
            return values.map(v => v === formulaSuffix);
        case "!=":
            return values.map(v => v !== formulaSuffix);
        case ">":
            return values.map(v => parseFloat(v) > parseFloat(formulaSuffix));
        case ">=":
            return values.map(v => parseFloat(v) >= parseFloat(formulaSuffix));
        case "<":
            return values.map(v => parseFloat(v) < parseFloat(formulaSuffix));
        case "<=":
            return values.map(v => parseFloat(v) <= parseFloat(formulaSuffix));
        case "in":
            if (formulaSuffix.startsWith("[") && formulaSuffix.endsWith("]") && formulaSuffix.length > 2) {
                formulaSuffix = formulaSuffix.substring(1, formulaSuffix.length - 1);
                const options = formulaSuffix.split(",").map(o => trimQuotes(o.trim()));
                return values.map(v => options.includes(v));
            }
            else {
                return [];
            }
    }
}
function trimQuotes(value) {
    if ((value.startsWith("'") && value.endsWith("'")) || (value.startsWith('"') && value.endsWith('"'))) {
        return value.substring(1, value.length - 1);
    }
    else {
        return value;
    }
}
function getFormulaInputNames(formulaPrefix) {
    const functions = ["any", "all"];
    for (let f of functions) {
        if (formulaPrefix.startsWith(f)) {
            return formulaPrefix.substring(4, formulaPrefix.length - 1).split(",").map(i => i.trim());
        }
    }
    return [formulaPrefix];
}
export function getFormElement(host) {
    let el = host;
    while (el) {
        if (el.tagName === "FORM") {
            return el;
        }
        el = el.parentElement;
        if (el.tagName === "BODY") {
            return undefined;
        }
    }
}
export function findInput(name) {
    return new Promise(resolve => {
        const input = document.querySelector(`[name='${name}']`);
        if (input) {
            resolve(input);
        }
        else {
            setTimeout(() => resolve(findInput(name)), 1000);
        }
    });
}
export function attachInputListener(name, callback, type = "inputUpdated") {
    return findInput(name).then((input) => {
        input.addEventListener(type, callback);
        return input;
    });
}
export class InputBase {
    constructor() {
        this.hasAnswerToValidate = () => {
            return this.answer && this.answerCorrection && this.answerCorrection.displayOn === "submit";
        };
    }
    /***
     Lifecycle methods
     */
    componentWillLoad() {
        this.answer = getAnswerElement(this.host);
        this.answerCorrection = getAnswerCorrectionElement(this.host);
    }
    connectedCallback() {
        this.form = getFormElement(this.host);
        if (this.requireIf || this.displayIf || this.disableIf || this.valueFrom) {
            if (['loaded', 'interactive', 'complete'].includes(document.readyState)) {
                this.setupDependentInputs();
            }
            else {
                this.loadCallback = this.setupDependentInputs.bind(this);
                document.addEventListener("load", this.setupDependentInputs);
            }
        }
        // TODO: Modify the check to better check for input type
        if (this.displayOn && this.host.parentElement.tagName.startsWith("TASK-INPUT")) {
            this.handleParentElementUpdate();
            this.displayOnCallback = this.handleParentElementUpdate.bind(this);
            this.host.parentElement.addEventListener("inputUpdated", this.displayOnCallback);
        }
    }
    disconnectedCallback() {
        if (this.loadCallback) {
            document.removeEventListener("load", this.loadCallback);
        }
        if (this.formCallback) {
            this.form.removeEventListener("input", this.formCallback);
        }
        if (this.displayOnCallback) {
            this.host.parentElement.removeEventListener("inputUpdated", this.displayOnCallback);
        }
        if (this.fromInput && this.fromInputUpdated) {
            // this.fromInput.removeEventListener("inputUpdated", this.fromInputUpdated)
        }
    }
    formUpdated() {
        if (this.form) {
            if (this.requireIf) {
                const result = evaluateFormula(this.requireIf, this.form);
                if (result !== undefined) {
                    this.required = result;
                }
            }
            if (this.displayIf) {
                const result = evaluateFormula(this.displayIf, this.form);
                if (result !== undefined) {
                    const previous = this.hidden;
                    this.hidden = !result;
                    if (this.hidden !== previous) {
                        this.inputUpdated.emit(this.form);
                    }
                }
            }
            if (this.disableIf) {
                const result = evaluateFormula(this.disableIf, this.form);
                if (result !== undefined) {
                    const previous = this.disabled;
                    this.disabled = result;
                    if (this.disabled !== previous) {
                        this.inputUpdated.emit(this.form);
                    }
                }
            }
        }
    }
    setupDependentInputs() {
        if ((this.requireIf || this.displayIf || this.disableIf) && this.form) {
            this.formCallback = this.formUpdated.bind(this);
            this.form.addEventListener('input', this.formCallback);
            this.formUpdated();
        }
        if (this.valueFrom) {
            attachInputListener(this.valueFrom, this.fromInputUpdated)
                .then(input => {
                this.fromInput = input;
                this.fromInputUpdated();
            });
        }
    }
    fromInputUpdated() {
        if (this.fromInput) {
            this.value = this.fromInput.value;
        }
    }
    handleParentElementUpdate() {
        const parent = this.host.parentElement;
        const previous = this.hidden;
        this.hidden = parent.value !== this.displayOn || parent.hidden;
        if (this.hidden !== previous) {
            this.inputUpdated.emit(this.form);
        }
    }
    //@Method()
    async readyToSubmit() {
        return Boolean(this.value);
    }
    //Method()
    async validateAgainstAnswer() {
        return !(this.hasAnswerToValidate() && this.value !== this.answer.value);
    }
    //@Method()
    async setShowCorrections(value) {
        if (this.hasAnswerToValidate()) {
            this.preventChanges = this.answerCorrection.preventChanges && value;
            if (this.value !== this.answer.value) {
                this.answerCorrection.displayCorrection = value;
                if (value) {
                    this.inputUpdated.emit(this.form);
                }
            }
        }
    }
    //@Watch("value")
    //@Watch("required")
    handleValueUpdate(_value) {
        this.inputUpdated.emit(this.form);
    }
    displayOnCallback(_ev) { }
    formCallback(_ev) { }
    loadCallback(_ev) { }
}
//# sourceMappingURL=inputBase.js.map
